CKEDITOR.plugins.setLang('youtube', 'ru', {
	button : 'Вставить YouTube видео',
	title : 'Вставить YouTube видео',
	txtEmbed : 'Вставьте HTML-код сюда',
	txtUrl : 'Вставьте адрес видео (URL)',
	txtWidth : 'Ширина',
	txtHeight : 'Высота',
	chkRelated : 'Показать похожие видео после завершения просмотра',
	txtStartAt : 'Начать с (сс или мм:сс или чч:мм:сс)',
	chkPrivacy : 'Включить режим повышенной конфиденциальности',
	chkOlderCode : 'Использовать старый код вставки',
	chkAutoplay: 'Автозапуск',
	chkControls: 'Показать панель управления',
	noCode : 'Вы должны ввести HTML-код или адрес',
	invalidEmbed : 'Ваш HTML-код не похож на правильный',
	invalidUrl : 'Ваш адрес видео не похож на правильный',
	or : 'или',
	noWidth : 'Вы должны указать ширину',
	invalidWidth : 'Укажите правильную ширину',
	noHeight : 'Вы должны указать высоту',
	invalidHeight : 'Укажите правильную высоту',
	invalidTime : 'Укажите правильное время начала',
	txtResponsive : 'Растягиваемое видео',
	txtNoEmbed : 'Не встраивать видео (обложка-ссылка на YouTube)'
});
